@@ -1,33 +1,41 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-
-from base64 import b64encode, b64decode
-from collections import OrderedDict
-from os.path import isfile
+import bisect
 import datetime
+import importlib
 import json
 import os
-import pkg_resources
-import time
-import bisect
-import warnings
 import re
-
+import time
+import warnings
+from base64 import b64decode
+from base64 import b64encode
+from collections import defaultdict
+from collections import OrderedDict
+from functools import lru_cache
 from html import escape
+from os.path import isfile
 
-try:
-    from ansi2html import Ansi2HTMLConverter, style
-
-    ANSI = True
-except ImportError:
-    # ansi2html is not installed
-    ANSI = False
-
-from py.xml import html, raw
+import pkg_resources
+import pytest
+from _pytest.logging import _remove_ansi_escape_sequences
+from py.xml import html
+from py.xml import raw
 
+from . import __pypi_url__
+from . import __version__
 from . import extras
-from . import __version__, __pypi_url__
+
+
+@lru_cache()
+def ansi_support():
+    try:
+        # from ansi2html import Ansi2HTMLConverter, style  # NOQA
+        return importlib.import_module("ansi2html")
+    except ImportError:
+        # ansi2html is not installed
+        pass
 
 
 def pytest_addhooks(pluginmanager):
@@ -62,6 +70,18 @@ def pytest_addoption(parser):
         default=[],
         help="append given css file content to report style file.",
     )
+    parser.addini(
+        "render_collapsed",
+        type="bool",
+        default=False,
+        help="Open the report with all rows collapsed. Useful for very large reports",
+    )
+    parser.addini(
+        "max_asset_filename_length",
+        default=255,
+        help="set the maximum filename length for assets "
+        "attached to the html report.",
+    )
 
 
 def pytest_configure(config):
@@ -69,9 +89,9 @@ def pytest_configure(config):
     if htmlpath:
         for csspath in config.getoption("css"):
             if not os.path.exists(csspath):
-                raise IOError(f"No such file or directory: '{csspath}'")
-        if not hasattr(config, "slaveinput"):
-            # prevent opening htmlpath on slave nodes (xdist)
+                raise OSError(f"No such file or directory: '{csspath}'")
+        if not hasattr(config, "workerinput"):
+            # prevent opening htmlpath on worker nodes (xdist)
             config._html = HTMLReport(htmlpath, config)
             config.pluginmanager.register(config._html)
 
@@ -83,6 +103,33 @@ def pytest_unconfigure(config):
         config.pluginmanager.unregister(html)
 
 
+@pytest.hookimpl(tryfirst=True, hookwrapper=True)
+def pytest_runtest_makereport(item, call):
+    outcome = yield
+    report = outcome.get_result()
+    if report.when == "call":
+        fixture_extras = getattr(item.config, "extras", [])
+        plugin_extras = getattr(report, "extra", [])
+        report.extra = fixture_extras + plugin_extras
+
+
+@pytest.fixture
+def extra(pytestconfig):
+    """Add details to the HTML reports.
+
+    .. code-block:: python
+
+        import pytest_html
+
+
+        def test_foo(extra):
+            extra.append(pytest_html.extras.url("http://www.example.com/"))
+    """
+    pytestconfig.extras = []
+    yield pytestconfig.extras
+    del pytestconfig.extras[:]
+
+
 def data_uri(content, mime_type="text/plain", charset="utf-8"):
     data = b64encode(content.encode(charset)).decode("ascii")
     return f"data:{mime_type};charset={charset};base64,{data}"
@@ -93,6 +140,7 @@ class HTMLReport:
         logfile = os.path.expanduser(os.path.expandvars(logfile))
         self.logfile = os.path.abspath(logfile)
         self.test_logs = []
+        self.title = os.path.basename(self.logfile)
         self.results = []
         self.errors = self.failed = 0
         self.passed = self.skipped = 0
@@ -101,6 +149,7 @@ class HTMLReport:
         self.rerun = 0 if has_rerun else None
         self.self_contained = config.getoption("self_contained_html")
         self.config = config
+        self.reports = defaultdict(list)
 
     class TestResult:
         def __init__(self, outcome, report, logfile, config):
@@ -112,6 +161,9 @@ class HTMLReport:
             self.additional_html = []
             self.links_html = []
             self.self_contained = config.getoption("self_contained_html")
+            self.max_asset_filename_length = int(
+                config.getini("max_asset_filename_length")
+            )
             self.logfile = logfile
             self.config = config
             self.row_table = self.row_extra = None
@@ -121,7 +173,12 @@ class HTMLReport:
             for extra_index, extra in enumerate(getattr(report, "extra", [])):
                 self.append_extra_html(extra, extra_index, test_index)
 
-            self.append_log_html(report, self.additional_html)
+            self.append_log_html(
+                report,
+                self.additional_html,
+                config.option.capture,
+                config.option.showcapture,
+            )
 
             cells = [
                 html.td(self.outcome, class_="col-result"),
@@ -137,9 +194,13 @@ class HTMLReport:
             )
 
             if len(cells) > 0:
+                tr_class = None
+                if self.config.getini("render_collapsed"):
+                    tr_class = "collapsed"
                 self.row_table = html.tr(cells)
                 self.row_extra = html.tr(
-                    html.td(self.additional_html, class_="extra", colspan=len(cells))
+                    html.td(self.additional_html, class_="extra", colspan=len(cells)),
+                    class_=tr_class,
                 )
 
         def __lt__(self, other):
@@ -157,19 +218,17 @@ class HTMLReport:
         def create_asset(
             self, content, extra_index, test_index, file_extension, mode="w"
         ):
-            # 255 is the common max filename length on various filesystems
             asset_file_name = "{}_{}_{}.{}".format(
                 re.sub(r"[^\w\.]", "_", self.test_id),
                 str(extra_index),
                 str(test_index),
                 file_extension,
-            )[-255:]
+            )[-self.max_asset_filename_length :]
             asset_path = os.path.join(
                 os.path.dirname(self.logfile), "assets", asset_file_name
             )
 
-            if not os.path.exists(os.path.dirname(asset_path)):
-                os.makedirs(os.path.dirname(asset_path))
+            os.makedirs(os.path.dirname(asset_path), exist_ok=True)
 
             relative_path = f"assets/{asset_file_name}"
 
@@ -180,39 +239,13 @@ class HTMLReport:
 
         def append_extra_html(self, extra, extra_index, test_index):
             href = None
-            if extra.get("format") == extras.FORMAT_IMAGE:
-                content = extra.get("content")
-                try:
-                    is_uri_or_path = content.startswith(("file", "http")) or isfile(
-                        content
-                    )
-                except ValueError:
-                    # On Windows, os.path.isfile throws this exception when
-                    # passed a b64 encoded image.
-                    is_uri_or_path = False
-                if is_uri_or_path:
-                    if self.self_contained:
-                        warnings.warn(
-                            "Self-contained HTML report "
-                            "includes link to external "
-                            "resource: {}".format(content)
-                        )
-                    html_div = html.a(html.img(src=content), href=content)
-                elif self.self_contained:
-                    src = "data:{};base64,{}".format(extra.get("mime_type"), content)
-                    html_div = html.img(src=src)
-                else:
-                    content = b64decode(content.encode("utf-8"))
-                    href = src = self.create_asset(
-                        content, extra_index, test_index, extra.get("extension"), "wb"
-                    )
-                    html_div = html.a(html.img(src=src), href=href)
-                self.additional_html.append(html.div(html_div, class_="image"))
+            if extra.get("format_type") == extras.FORMAT_IMAGE:
+                self._append_image(extra, extra_index, test_index)
 
-            elif extra.get("format") == extras.FORMAT_HTML:
+            elif extra.get("format_type") == extras.FORMAT_HTML:
                 self.additional_html.append(html.div(raw(extra.get("content"))))
 
-            elif extra.get("format") == extras.FORMAT_JSON:
+            elif extra.get("format_type") == extras.FORMAT_JSON:
                 content = json.dumps(extra.get("content"))
                 if self.self_contained:
                     href = data_uri(content, mime_type=extra.get("mime_type"))
@@ -221,7 +254,7 @@ class HTMLReport:
                         content, extra_index, test_index, extra.get("extension")
                     )
 
-            elif extra.get("format") == extras.FORMAT_TEXT:
+            elif extra.get("format_type") == extras.FORMAT_TEXT:
                 content = extra.get("content")
                 if isinstance(content, bytes):
                     content = content.decode("utf-8")
@@ -232,24 +265,31 @@ class HTMLReport:
                         content, extra_index, test_index, extra.get("extension")
                     )
 
-            elif extra.get("format") == extras.FORMAT_URL:
+            elif extra.get("format_type") == extras.FORMAT_URL:
                 href = extra.get("content")
 
+            elif extra.get("format_type") == extras.FORMAT_VIDEO:
+                self._append_video(extra, extra_index, test_index)
+
             if href is not None:
                 self.links_html.append(
                     html.a(
                         extra.get("name"),
-                        class_=extra.get("format"),
+                        class_=extra.get("format_type"),
                         href=href,
                         target="_blank",
                     )
                 )
                 self.links_html.append(" ")
 
-        def append_log_html(self, report, additional_html):
-            log = html.div(class_="log")
+        def _populate_html_log_div(self, log, report):
             if report.longrepr:
-                for line in report.longreprtext.splitlines():
+                # longreprtext is only filled out on failure by pytest
+                #    otherwise will be None.
+                #  Use full_text if longreprtext is None-ish
+                #   we added full_text elsewhere in this file.
+                text = report.longreprtext or report.full_text
+                for line in text.splitlines():
                     separator = line.startswith("_ " * 10)
                     if separator:
                         log.append(line[:80])
@@ -265,17 +305,90 @@ class HTMLReport:
                 header, content = map(escape, section)
                 log.append(f" {header:-^80} ")
                 log.append(html.br())
-                if ANSI:
-                    converter = Ansi2HTMLConverter(inline=False, escaped=False)
+
+                if ansi_support():
+                    converter = ansi_support().Ansi2HTMLConverter(
+                        inline=False, escaped=False
+                    )
                     content = converter.convert(content, full=False)
+                else:
+                    content = _remove_ansi_escape_sequences(content)
+
                 log.append(raw(content))
                 log.append(html.br())
 
+        def append_log_html(
+            self,
+            report,
+            additional_html,
+            pytest_capture_value,
+            pytest_show_capture_value,
+        ):
+            log = html.div(class_="log")
+
+            should_skip_captured_output = pytest_capture_value == "no"
+            if report.outcome == "failed" and not should_skip_captured_output:
+                should_skip_captured_output = pytest_show_capture_value == "no"
+            if not should_skip_captured_output:
+                self._populate_html_log_div(log, report)
+
             if len(log) == 0:
                 log = html.div(class_="empty log")
                 log.append("No log output captured.")
+
             additional_html.append(log)
 
+        def _make_media_html_div(
+            self, extra, extra_index, test_index, base_extra_string, base_extra_class
+        ):
+            content = extra.get("content")
+            try:
+                is_uri_or_path = content.startswith(("file", "http")) or isfile(content)
+            except ValueError:
+                # On Windows, os.path.isfile throws this exception when
+                # passed a b64 encoded image.
+                is_uri_or_path = False
+            if is_uri_or_path:
+                if self.self_contained:
+                    warnings.warn(
+                        "Self-contained HTML report "
+                        "includes link to external "
+                        f"resource: {content}"
+                    )
+
+                html_div = html.a(
+                    raw(base_extra_string.format(extra.get("content"))), href=content
+                )
+            elif self.self_contained:
+                src = f"data:{extra.get('mime_type')};base64,{content}"
+                html_div = raw(base_extra_string.format(src))
+            else:
+                content = b64decode(content.encode("utf-8"))
+                href = src = self.create_asset(
+                    content, extra_index, test_index, extra.get("extension"), "wb"
+                )
+                html_div = html.a(
+                    raw(base_extra_string.format(src)),
+                    class_=base_extra_class,
+                    target="_blank",
+                    href=href,
+                )
+            return html_div
+
+        def _append_image(self, extra, extra_index, test_index):
+            image_base = '<img src="{}"/>'
+            html_div = self._make_media_html_div(
+                extra, extra_index, test_index, image_base, "image"
+            )
+            self.additional_html.append(html.div(html_div, class_="image"))
+
+        def _append_video(self, extra, extra_index, test_index):
+            video_base = '<video controls><source src="{}" type="video/mp4"></video>'
+            html_div = self._make_media_html_div(
+                extra, extra_index, test_index, video_base, "video"
+            )
+            self.additional_html.append(html.div(html_div, class_="video"))
+
     def _appendrow(self, outcome, report):
         result = self.TestResult(outcome, report, self.logfile, self.config)
         if result.row_table is not None:
@@ -334,13 +447,13 @@ class HTMLReport:
             __name__, os.path.join("resources", "style.css")
         ).decode("utf-8")
 
-        if ANSI:
+        if ansi_support():
             ansi_css = [
                 "\n/******************************",
                 " * ANSI2HTML STYLES",
                 " ******************************/\n",
             ]
-            ansi_css.extend([str(r) for r in style.get_styles()])
+            ansi_css.extend([str(r) for r in ansi_support().style.get_styles()])
             self.style_css += "\n".join(ansi_css)
 
         # <DF> Add user-provided CSS
@@ -349,7 +462,7 @@ class HTMLReport:
             self.style_css += "\n * CUSTOM CSS"
             self.style_css += f"\n * {path}"
             self.style_css += "\n ******************************/\n\n"
-            with open(path, "r") as f:
+            with open(path) as f:
                 self.style_css += f.read()
 
         css_href = "assets/style.css"
@@ -425,7 +538,7 @@ class HTMLReport:
             html.th("Result", class_="sortable result initial-sort", col="result"),
             html.th("Test", class_="sortable", col="name"),
             html.th("Duration", class_="sortable numeric", col="duration"),
-            html.th("Links"),
+            html.th("Links", class_="sortable links", col="links"),
         ]
         session.config.hook.pytest_html_results_table_header(cells=cells)
 
@@ -457,9 +570,11 @@ class HTMLReport:
             __name__, os.path.join("resources", "main.js")
         ).decode("utf-8")
 
+        session.config.hook.pytest_html_report_title(report=self)
+
         body = html.body(
             html.script(raw(main_js)),
-            html.h1(os.path.basename(self.logfile)),
+            html.h1(self.title),
             html.p(
                 "Report generated on {} at {} by ".format(
                     generated.strftime("%d-%b-%Y"), generated.strftime("%H:%M:%S")
@@ -505,8 +620,12 @@ class HTMLReport:
             if isinstance(value, str) and value.startswith("http"):
                 value = html.a(value, href=value, target="_blank")
             elif isinstance(value, (list, tuple, set)):
-                value = ", ".join(str(i) for i in value)
-            rows.append(html.tr(html.td(key), html.td(value)))
+                value = ", ".join(str(i) for i in sorted(map(str, value)))
+            elif isinstance(value, dict):
+                sorted_dict = {k: value[k] for k in sorted(value)}
+                value = json.dumps(sorted_dict)
+            raw_value_string = raw(str(value))
+            rows.append(html.tr(html.td(key), html.td(raw_value_string)))
 
         environment.append(html.table(rows, id="environment"))
         return environment
@@ -515,10 +634,9 @@ class HTMLReport:
         dir_name = os.path.dirname(self.logfile)
         assets_dir = os.path.join(dir_name, "assets")
 
-        if not os.path.exists(dir_name):
-            os.makedirs(dir_name)
-        if not self.self_contained and not os.path.exists(assets_dir):
-            os.makedirs(assets_dir)
+        os.makedirs(dir_name, exist_ok=True)
+        if not self.self_contained:
+            os.makedirs(assets_dir, exist_ok=True)
 
         with open(self.logfile, "w", encoding="utf-8") as f:
             f.write(report_content)
@@ -527,15 +645,66 @@ class HTMLReport:
             with open(style_path, "w", encoding="utf-8") as f:
                 f.write(self.style_css)
 
+    def _post_process_reports(self):
+        for test_name, test_reports in self.reports.items():
+            outcome = "passed"
+            wasxfail = False
+            failure_when = None
+            full_text = ""
+            extras = []
+            duration = 0.0
+
+            # in theory the last one should have all logs so we just go
+            #  through them all to figure out the outcome, xfail, duration,
+            #    extras, and when it swapped from pass
+            for test_report in test_reports:
+                full_text += test_report.longreprtext
+                extras.extend(getattr(test_report, "extra", []))
+                duration += getattr(test_report, "duration", 0.0)
+
+                if (
+                    test_report.outcome not in ("passed", "rerun")
+                    and outcome == "passed"
+                ):
+                    outcome = test_report.outcome
+                    failure_when = test_report.when
+
+                if hasattr(test_report, "wasxfail"):
+                    wasxfail = True
+
+                if test_report.outcome == "rerun":
+                    self.append_other(test_report)
+
+            # the following test_report.<X> = settings come at the end of us
+            #  looping through all test_reports that make up a single
+            #    case.
+
+            # outcome on the right comes from the outcome of the various
+            #  test_reports that make up this test case
+            #    we are just carrying it over to the final report.
+            test_report.outcome = outcome
+            test_report.when = "call"
+            test_report.nodeid = test_name
+            test_report.longrepr = full_text
+            test_report.extra = extras
+            test_report.duration = duration
+
+            if wasxfail:
+                test_report.wasxfail = True
+
+            if test_report.outcome == "passed":
+                self.append_passed(test_report)
+            elif test_report.outcome == "skipped":
+                self.append_skipped(test_report)
+            elif test_report.outcome == "failed":
+                test_report.when = failure_when
+                self.append_failed(test_report)
+
+            # we don't append other here since the only case supported
+            #  for append_other is rerun, which is handled in the loop above
+
     def pytest_runtest_logreport(self, report):
-        if report.passed:
-            self.append_passed(report)
-        elif report.failed:
-            self.append_failed(report)
-        elif report.skipped:
-            self.append_skipped(report)
-        else:
-            self.append_other(report)
+        self.reports[report.nodeid].append(report)
 
     def pytest_collectreport(self, report):
         if report.failed:
@@ -545,6 +714,7 @@ class HTMLReport:
         self.suite_start_time = time.time()
 
     def pytest_sessionfinish(self, session):
+        self._post_process_reports()
         report_content = self._generate_report(session)
         self._save_report(report_content)
 
