@@ -1,33 +1,42 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-
-from base64 import b64encode
+import builtins
 import json
 import os
-import pkg_resources
 import random
 import re
+from base64 import b64encode
 
+import pkg_resources
 import pytest
 
 pytest_plugins = ("pytester",)
 
+if os.name == "nt":
+    # Force a utf-8 encoding on file io (since by default windows does not). See
+    # https://github.com/pytest-dev/pytest-html/issues/336
+    #  If we drop support for Python 3.6 and earlier could use python -X utf8 instead.
+    _real_open = builtins.open
 
-def handle_tr_writer_deprecation():
-    # Remove this function when they've fixed
+    def _open(file, mode="r", buffering=-1, encoding=None, *args, **kwargs):
+        if mode in ("r", "w") and encoding is None:
+            encoding = "utf-8"
+
+        return _real_open(file, mode, buffering, encoding, *args, **kwargs)
+
+    builtins.open = _open
+
+
+def remove_deprecation_from_recwarn(recwarn):
+    # TODO: Temporary hack until they fix
     # https://github.com/pytest-dev/pytest/issues/6936
-    import warnings
-    from _pytest.warnings import _setoption
-
-    arg = "ignore:TerminalReporter.writer:pytest.PytestDeprecationWarning"
-    _setoption(warnings, arg)
+    return [
+        item for item in recwarn if "TerminalReporter.writer" not in repr(item.message)
+    ]
 
 
 def run(testdir, path="report.html", *args):
-    # TODO: Temporary hack until they fix
-    # https://github.com/pytest-dev/pytest/issues/6936
-    handle_tr_writer_deprecation()  # TODO: Temporary hack
     path = testdir.tmpdir.join(path)
     result = testdir.runpytest("--html", path, *args)
     return result, read_html(path)
@@ -232,9 +241,6 @@ class TestHTML:
         assert report_title in html
 
     def test_report_title_addopts_env_var(self, testdir, monkeypatch):
-        # TODO: Temporary hack until they fix
-        # https://github.com/pytest-dev/pytest/issues/6936
-        handle_tr_writer_deprecation()
         report_location = "REPORT_LOCATION"
         report_name = "MuhReport"
         monkeypatch.setenv(report_location, report_name)
@@ -510,7 +516,9 @@ class TestHTML:
         assert result.ret == 0
         src = f"assets/test_extra_image_separated.py__test_pass_0_0.{file_extension}"
         link = f'<a class="image" href="{src}" target="_blank">'
+        img = f'<img src="{src}"/>'
         assert link in html
+        assert img in html
         assert os.path.exists(src)
 
     @pytest.mark.parametrize(
@@ -544,8 +552,10 @@ class TestHTML:
             asset_name = "test_extra_image_separated_rerun.py__test_fail"
             src = f"assets/{asset_name}_0_{i}.{file_extension}"
             link = f'<a class="image" href="{src}" target="_blank">'
+            img = f'<img src="{src}"/>'
             assert result.ret
             assert link in html
+            assert img in html
             assert os.path.exists(src)
 
     @pytest.mark.parametrize("src_type", ["https://", "file://", "image.png"])
@@ -570,7 +580,8 @@ class TestHTML:
         assert result.ret == 0
         assert '<a href="{0}"><img src="{0}"/>'.format(content) in html
 
-    def test_very_long_test_name(self, testdir):
+    @pytest.mark.parametrize("max_asset_filename_length", [10, 100])
+    def test_very_long_test_name(self, testdir, max_asset_filename_length):
         testdir.makeconftest(
             """
             import pytest
@@ -591,12 +602,22 @@ class TestHTML:
                 assert False
         """
         )
-        result, html = run(testdir)
-        file_name = f"test_very_long_test_name.py__{test_name}_0_0.png"[-255:]
+        testdir.makeini(
+            f"""
+            [pytest]
+            max_asset_filename_length = {max_asset_filename_length}
+        """
+        )
+        result, html = run(testdir, "report.html")
+        file_name = f"test_very_long_test_name.py__{test_name}_0_0.png"[
+            -max_asset_filename_length:
+        ]
         src = "assets/" + file_name
         link = f'<a class="image" href="{src}" target="_blank">'
+        img = f'<img src="{src}"/>'
         assert result.ret
         assert link in html
+        assert img in html
         assert os.path.exists(src)
 
     def test_extra_fixture(self, testdir):
@@ -776,7 +797,7 @@ class TestHTML:
         assert "Environment" in html
         assert len(re.findall("ZZZ.+AAA", html, re.DOTALL)) == 1
 
-    def test_xdist_crashing_slave(self, testdir):
+    def test_xdist_crashing_worker(self, testdir):
         """https://github.com/pytest-dev/pytest-html/issues/21"""
         testdir.makepyfile(
             """
@@ -802,14 +823,14 @@ class TestHTML:
         assert result.ret == 0
         assert_results(html, passed=1)
 
-    def test_ansi_color(self, testdir):
-        try:
-            import ansi2html  # NOQA
+    @pytest.mark.parametrize(
+        "with_ansi", [True, False],
+    )
+    def test_ansi_color(self, testdir, mocker, with_ansi):
+        if not with_ansi:
+            mock_ansi_support = mocker.patch("pytest_html.plugin.ansi_support")
+            mock_ansi_support.return_value = None
 
-            ANSI = True
-        except ImportError:
-            # ansi2html is not installed
-            ANSI = False
         pass_content = [
             '<span class="ansi31">RCOLOR',
             '<span class="ansi32">GCOLOR',
@@ -827,11 +848,34 @@ class TestHTML:
         result, html = run(testdir, "report.html", "--self-contained-html")
         assert result.ret == 0
         for content in pass_content:
-            if ANSI:
+            if with_ansi:
                 assert content in html
             else:
                 assert content not in html
 
+    def test_ansi_escape_sequence_removed(self, testdir):
+        testdir.makeini(
+            r"""
+            [pytest]
+            log_cli = 1
+            log_cli_level = INFO
+        """
+        )
+        testdir.makepyfile(
+            r"""
+            import logging
+            logging.basicConfig()
+            LOGGER = logging.getLogger()
+            def test_ansi():
+                LOGGER.info("ANSI removed")
+        """
+        )
+        result, html = run(
+            testdir, "report.html", "--self-contained-html", "--color=yes"
+        )
+        assert result.ret == 0
+        assert not re.search(r"\[[\d;]+m", html)
+
     @pytest.mark.parametrize("content", [("'foo'"), ("u'\u0081'")])
     def test_utf8_longrepr(self, testdir, content):
         testdir.makeconftest(
@@ -881,7 +925,8 @@ class TestHTML:
             cssargs.extend(["--css", path])
         result, html = run(testdir, "report.html", "--self-contained-html", *cssargs)
         assert result.ret == 0
-        assert len(recwarn) == 0
+        warnings = remove_deprecation_from_recwarn(recwarn)
+        assert len(warnings) == 0
         for k, v in css.items():
             assert str(v["path"]) in html
             assert v["style"] in html
@@ -894,9 +939,6 @@ class TestHTML:
         assert "No such file or directory: 'style.css'" in result.stderr.str()
 
     def test_css_invalid_no_html(self, testdir):
-        # TODO: Temporary hack until they fix
-        # https://github.com/pytest-dev/pytest/issues/6936
-        handle_tr_writer_deprecation()
         testdir.makepyfile("def test_pass(): pass")
         result = testdir.runpytest("--css", "style.css")
         assert result.ret == 0
@@ -953,3 +995,118 @@ class TestHTML:
         result, html = run(testdir)
         assert result.ret == 0
         assert len(re.findall(content_report_title, html)) == 1
+
+    def test_setup_and_teardown_in_html(self, testdir):
+        testdir.makepyfile(
+            """
+            import pytest
+            @pytest.fixture(scope="function")
+            def setupAndTeardown():
+                print ("this is setup")
+                yield
+                print ("this is teardown")
+
+            def test_setup_and_teardown(setupAndTeardown):
+                print ("this is the test case")
+        """
+        )
+        result, html = run(testdir)
+        assert result.ret == 0
+        assert_results(html, tests=1, passed=1)
+        assert "this is setup" in html
+        assert "this is teardown" in html
+        assert "this is the test case" in html
+
+    def test_setup_failures_are_errors(self, testdir):
+        testdir.makepyfile(
+            """
+            import pytest
+            @pytest.fixture(scope="function")
+            def setup():
+                assert 0, "failure!"
+
+            def test_setup(setup):
+                print ("this is the test case")
+        """
+        )
+        result, html = run(testdir)
+        assert result.ret == 1
+        assert_results(html, tests=0, passed=0, errors=1)
+        assert "this is the test case" not in html
+
+    def test_teardown_failures_are_errors(self, testdir):
+        testdir.makepyfile(
+            """
+            import pytest
+            @pytest.fixture(scope="function")
+            def teardown():
+                yield
+                assert 0, "failure!"
+
+            def test_setup(teardown):
+                print ("this is the test case")
+        """
+        )
+        result, html = run(testdir)
+        assert result.ret == 1
+        assert_results(html, tests=0, passed=0, errors=1)
+        assert "this is the test case" in html
+
+    @pytest.mark.parametrize(
+        "capture_flag, should_capture",
+        [("-s", False), ("--capture=no", False), ("--capture=sys", True)],
+    )
+    def test_extra_log_reporting_respects_capture_no(
+        self, testdir, capture_flag, should_capture
+    ):
+        testdir.makepyfile(
+            """
+            import sys
+            def test_capture_no():
+                print("stdout print line")
+                print("stderr print line", file=sys.stderr)
+        """
+        )
+
+        result, html = run(testdir, "report.html", capture_flag)
+        assert result.ret == 0
+        assert_results(html)
+
+        extra_log_div_regex = re.compile(
+            '<div class="log"> -+Captured stdout call-+ <br/>stdout print line\n<br/> '
+            "-+Captured stderr call-+ <br/>stderr print line\n<br/></div>"
+        )
+        if should_capture:
+            assert extra_log_div_regex.search(html) is not None
+        else:
+            assert extra_log_div_regex.search(html) is None
+
+    @pytest.mark.parametrize(
+        "show_capture_flag, should_capture",
+        [("--show-capture=no", False), ("--show-capture=all", True)],
+    )
+    def test_extra_log_reporting_respects_show_capture_no(
+        self, testdir, show_capture_flag, should_capture
+    ):
+        testdir.makepyfile(
+            """
+            import sys
+            def test_show_capture_no():
+                print("stdout print line")
+                print("stderr print line", file=sys.stderr)
+                assert False
+        """
+        )
+
+        result, html = run(testdir, "report.html", show_capture_flag)
+        assert result.ret == 1
+        assert_results(html, passed=0, failed=1)
+
+        extra_log_div_regex = re.compile(
+            '<div class="log">.*-+Captured stdout call-+ <br/>stdout print line\n<br/> '
+            "-+Captured stderr call-+ <br/>stderr print line\n<br/></div>"
+        )
+        if should_capture:
+            assert extra_log_div_regex.search(html) is not None
+        else:
+            assert extra_log_div_regex.search(html) is None
